/*
 * Sonar, entreprise quality control tool.
 * Copyright (C) 2007-2008 Hortis-GRC SA
 * mailto:be_agile HAT hortis DOT ch
 *
 * Sonar is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with {library}; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package ch.hortis.sonar.web.charts;

import org.jfree.chart.JFreeChart;
import org.jfree.chart.axis.DateAxis;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.renderer.xy.XYLineAndShapeRenderer;
import org.jfree.chart.renderer.xy.StandardXYItemRenderer;
import org.jfree.chart.title.TextTitle;
import org.jfree.data.time.Day;
import org.jfree.data.time.TimeSeries;
import org.jfree.data.time.TimeSeriesCollection;
import org.jfree.data.xy.XYDataset;
import org.jfree.ui.RectangleEdge;

import java.awt.*;
import java.awt.image.BufferedImage;
import java.io.IOException;
import java.text.DecimalFormat;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.TreeMap;

import org.jfree.chart.axis.AxisLocation;
import org.jfree.chart.plot.Marker;
import org.jfree.chart.plot.ValueMarker;
import org.jfree.ui.RectangleAnchor;
import org.jfree.ui.TextAnchor;

public class HistoryChart2 extends BaseChart {

    private XYPlot plot;
    private TreeMap<Integer, TimeSeries> listSeries;
    private int percentAxisId = -1;
    
    public HistoryChart2(int width, int height) {
        super(width, height);

        listSeries = new TreeMap<Integer, TimeSeries>();
        plot = new XYPlot();

        DateAxis domainAxis = new DateAxis();
        domainAxis.setAutoRangeMinimumSize(86400000.0 * 1.1);
        domainAxis.setUpperMargin(0.1);

        plot.setDomainAxis(domainAxis);
    }

    public void initLegend(Integer id, String legend, Boolean isPercent) {

        TimeSeries series = new TimeSeries(legend);

        listSeries.put(id, series);

        TimeSeriesCollection timeSeriesColl = new TimeSeriesCollection();
        timeSeriesColl.addSeries(series);
        plot.setDataset(id, timeSeriesColl);

        // Gestion des axes
        if (isPercent) {
            if (percentAxisId == -1) {
                NumberAxis rangeAxis = new NumberAxis(legend);
                rangeAxis.setNumberFormatOverride(new DecimalFormat("0'%'"));
                rangeAxis.setStandardTickUnits(NumberAxis.createIntegerTickUnits());
                rangeAxis.setUpperBound(100.0);
                rangeAxis.setLowerBound(0.0);
                plot.setRangeAxisLocation(id, AxisLocation.TOP_OR_LEFT);
                plot.setRangeAxis(id, rangeAxis);
                plot.mapDatasetToRangeAxis(id, id);
                percentAxisId = id;
            } else {
                NumberAxis curAxis = (NumberAxis)plot.getRangeAxis(percentAxisId);
                curAxis.setLabel(curAxis.getLabel()+", "+legend);
                plot.mapDatasetToRangeAxis(id, percentAxisId);
            }
        } else {
            NumberAxis rangeAxis = new NumberAxis(legend);
            rangeAxis.setAutoRangeIncludesZero(false);
            rangeAxis.setStandardTickUnits(NumberAxis.createIntegerTickUnits());
            rangeAxis.setAutoRangeMinimumSize(2.0);
            plot.setRangeAxisLocation(id, AxisLocation.TOP_OR_RIGHT);
            plot.setRangeAxis(id, rangeAxis);
            plot.mapDatasetToRangeAxis(id, id);
        }


        XYLineAndShapeRenderer renderer = new XYLineAndShapeRenderer();
        renderer.setBaseShapesVisible(false);
        renderer.setBaseStroke(new BasicStroke( 2.0f ));
        plot.setRenderer(id, renderer);
    }

    public void addMeasure(Double value, Date date, Integer id) {
        listSeries.get(id).addOrUpdate(new Day(date), value);
    }

    public void addLabel(Date date, String label) throws ParseException {
        Day d = new Day(date);
        double millis = d.getFirstMillisecond();
        Marker marker = new ValueMarker(millis);
        marker.setLabel(label);
        marker.setLabelAnchor(RectangleAnchor.TOP_RIGHT);
        marker.setLabelTextAnchor(TextAnchor.TOP_LEFT);
        Color c = new Color(17,40,95);
        marker.setLabelPaint(c);
        marker.setPaint(c);
        marker.setStroke(new BasicStroke(1.0f, BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND, 3.0f, new float[]{ 5f, 5f, 10f, 5f}, 2.0f ));
        plot.addDomainMarker(marker);
    }

    protected BufferedImage getChartImage() throws IOException {
        JFreeChart chart = new JFreeChart(null, TextTitle.DEFAULT_FONT, plot, true);
        configureChart(chart, RectangleEdge.BOTTOM);
        return super.getBufferedImage(chart);
    }
}