/*
 * Sonar, entreprise quality control tool.
 * Copyright (C) 2007-2008 Hortis-GRC SA
 * mailto:be_agile HAT hortis DOT ch
 *
 * Sonar is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with Sonar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package org.sonar.web;

import org.apache.commons.configuration.CompositeConfiguration;
import org.apache.commons.configuration.Configuration;
import org.apache.commons.configuration.EnvironmentConfiguration;
import org.apache.commons.configuration.PropertiesConfiguration;
import org.apache.commons.configuration.SystemConfiguration;

import javax.servlet.ServletContextEvent;
import java.io.File;
import java.net.URISyntaxException;

public class ConfigurationFactory {
  public Configuration getConfiguration(ServletContextEvent sce) {
    CompositeConfiguration configuration = new CompositeConfiguration();
    configuration.addConfiguration(getConfigurationFromPropertiesFile());
    configuration.addConfiguration(new SystemConfiguration());
    configuration.addConfiguration(new EnvironmentConfiguration());
    configuration.addConfiguration(getDirectoriesConfiguration(configuration, sce));
    return configuration;
  }

  private Configuration getDirectoriesConfiguration(Configuration config, ServletContextEvent sce) {
    PropertiesConfiguration result = new PropertiesConfiguration();
    result.setProperty(WebProperties.ROOT_PUBLIC_DIR, autodetectWebappPublicDirectory(sce));
    if (config.getString(WebProperties.ROOT_DIR)==null) {
      result.setProperty(WebProperties.ROOT_DIR, autodetectRootDir());
    }
    return result;
  }

  private String autodetectRootDir() {
    try {
      return new File(getClass().getResource("/extensions").toURI()).getParent();

    } catch (URISyntaxException e) {
      e.printStackTrace();
      // TODO log
    }
    return null;
  }

  protected PropertiesConfiguration getConfigurationFromPropertiesFile(String filename) throws ConfigurationException {
    try {
      return new PropertiesConfiguration(ConfigurationFactory.class.getResource("/" + filename));
    } catch (org.apache.commons.configuration.ConfigurationException e) {
      throw new ConfigurationException("can not load the file " + filename + " from classpath", e);
    }
  }

  public PropertiesConfiguration getConfigurationFromPropertiesFile() throws ConfigurationException {
    return getConfigurationFromPropertiesFile("sonar.properties");
  }

  protected String autodetectWebappPublicDirectory(ServletContextEvent sce) {
    String webAppPublicDirPath = sce.getServletContext().getRealPath("/public/");
    if (webAppPublicDirPath == null) {
      throw new ConfigurationException("Web app directory not found : /public/");
    }
    File file = new File(webAppPublicDirPath);
    if (!file.exists()) {
      throw new ConfigurationException("Web app directory not found : " + file);
    }
    return file.toString();
  }
}
